{ config, pkgs, lib, ... }:
{
  options.lugnet = lib.mkOption {
    type = lib.types.attrs;
  };
  config = {
    lugnet = rec {
      hostToTincDef = _: h: ''
        Subnet = ${h.ipv4}
        Subnet = ${h.ipv6}
        ${lib.concatMapStringsSep "\n" (s: "Subnet = " + s) (h.subnets or [])}
        ${lib.concatMapStringsSep "\n" (a: "Address = " + a) (h.addresses or [])}
        ${lib.optionalString (h ? Ed25519PublicKey) "Ed25519PublicKey = ${h.Ed25519PublicKey}"}
        ${h.RSAPublicKey or ""}
      '';
      hosts = import ./vpn-hosts.nix;
      tincDefs = lib.mapAttrs hostToTincDef hosts;
      zoneDefs = lib.concatStringsSep "\n" (lib.mapAttrsToList (name: info: ''
        ${name} IN A ${info.ipv4}
                IN AAAA ${info.ipv6}
      '') hosts);
    };

    networking = {
      firewall.allowedTCPPorts = [ 655 ];
      firewall.allowedUDPPorts = [ 655 ];
      interfaces."tinc.lugnet" = let
          hostinfo = config.lugnet.hosts.${config.networking.hostName};
      in {
        ipv4.addresses = [ { address = hostinfo.ipv4; prefixLength = 24; } ];
        ipv6.addresses = [ { address = hostinfo.ipv6; prefixLength = 80; } ];
      };
      search = [ "sphalerite.tech" ];
    };
    services.tinc.networks.lugnet = {
      extraConfig = ''
        ConnectTo = lugn
        LocalDiscovery = yes
      '';
      hosts = config.lugnet.tincDefs;
    };
  };
}
